import connexion
import six

from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.execute import Execute  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api import util

from apitaxcore.models.State import State
from apitaxcore.models.Credentials import Credentials
from apitaxcore.models.Options import Options
from commandtax.flow.Connector import Connector
from apitax.api.utilities.Mappers import mapUserAuthToCredentials

import traceback

from apitaxcore.flow.responses.ApitaxResponse import ApitaxResponse
from apitax.api.utilities.Lifecycle import redirectIfUnauthorized, errorIfUnauthorized
from flask_jwt_extended import (create_access_token, create_refresh_token, jwt_required, jwt_refresh_token_required,
                                get_jwt_identity, get_raw_jwt, get_jwt_claims)


@jwt_required
def command(driver, execute=None):  # noqa: E501
    """Execute a Command

    Execute a command # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param execute: The data needed to execute this command
    :type execute: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        execute = Execute.from_dict(connexion.request.get_json())  # noqa: E501

    # TODO: Utilize the driver passed rather than the first string component

    response = errorIfUnauthorized(role='user')
    if response:
        return response

    try:
        parameters = {}

        if execute.command.parameters:
            parameters = execute.command.parameters

        credentials = Credentials()

        options = Options()

        if execute.command.options:
            if 'debug' in execute.command.options:
                options.debug = execute.command.options['debug']

            if 'sensitive' in execute.command.options:
                options.sensitive = execute.command.options['sensitive']

        if execute.auth:
            credentials = mapUserAuthToCredentials(execute.auth, credentials)
            if not execute.auth.api_token:
                options.sensitive = True

        connector = Connector(options=options, credentials=credentials, command=driver + " " + execute.command.command,
                              parameters=parameters)

        response: ApitaxResponse = connector.execute()

        response = Response(status=response.getResponseStatusCode(), body=response.getResponseBody())

        if options.debug:
            response.log = connector.logBuffer

        return response
    except:
        State.log.error(traceback.format_exc())
        State.log.getLoggerDriver().outputLog()
        if 'debug' in execute.command.options and execute.command.options['debug']:
            return ErrorResponse(status=500,
                                 message="Uncaught exception occured during processing. To get a larger stack trace, visit the logs.",
                                 state=traceback.format_exc())
        else:
            return ErrorResponse(status=500, message="")



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from apitax.api.models.base_model_ import Model
from apitax.api import util


class Response(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, status=None, message=None, body=None, log=None):  # noqa: E501
        """Response - a model defined in Swagger

        :param status: The status of this Response.  # noqa: E501
        :type status: int
        :param message: The message of this Response.  # noqa: E501
        :type message: str
        :param body: The body of this Response.  # noqa: E501
        :type body: object
        :param log: The log of this Response.  # noqa: E501
        :type log: str
        """
        self.swagger_types = {
            'status': int,
            'message': str,
            'body': object,
            'log': str
        }

        self.attribute_map = {
            'status': 'status',
            'message': 'message',
            'body': 'body',
            'log': 'log'
        }

        self._status = status
        self._message = message
        self._body = body
        self._log = log

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Response of this Response.  # noqa: E501
        :rtype: Response
        """
        return util.deserialize_model(dikt, cls)

    @property
    def status(self):
        """Gets the status of this Response.


        :return: The status of this Response.
        :rtype: int
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Response.


        :param status: The status of this Response.
        :type status: int
        """
        if status is None:
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def message(self):
        """Gets the message of this Response.


        :return: The message of this Response.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this Response.


        :param message: The message of this Response.
        :type message: str
        """

        self._message = message

    @property
    def body(self):
        """Gets the body of this Response.


        :return: The body of this Response.
        :rtype: object
        """
        return self._body

    @body.setter
    def body(self, body):
        """Sets the body of this Response.


        :param body: The body of this Response.
        :type body: object
        """
        if body is None:
            raise ValueError("Invalid value for `body`, must not be `None`")  # noqa: E501

        self._body = body

    @property
    def log(self):
        """Gets the log of this Response.


        :return: The log of this Response.
        :rtype: str
        """
        return self._log

    @log.setter
    def log(self, log):
        """Sets the log of this Response.


        :param log: The log of this Response.
        :type log: str
        """

        self._log = log



import connexion
import six

from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api.models.script_create import ScriptCreate  # noqa: E501
from apitax.api.models.script_delete import ScriptDelete  # noqa: E501
from apitax.api.models.script_rename import ScriptRename  # noqa: E501
from apitax.api.models.script_save import ScriptSave  # noqa: E501
from apitax.api import util

from apitaxcore.flow.LoadedDrivers import LoadedDrivers
from scriptax.drivers.Driver import Driver
from apitaxcore.flow.responses.ApitaxResponse import ApitaxResponse
from apitax.api.utilities.Lifecycle import redirectIfUnauthorized, errorIfUnauthorized
from flask_jwt_extended import (create_access_token, create_refresh_token, jwt_required, jwt_refresh_token_required,
                                get_jwt_identity, get_raw_jwt, get_jwt_claims)

@jwt_required
def create_driver_script(driver, script_create=None):  # noqa: E501
    """Create a new script

    Create a new script # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param script_create: The data needed to create this script
    :type script_create: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        script_create = ScriptCreate.from_dict(connexion.request.get_json())  # noqa: E501

    response = errorIfUnauthorized(role='developer')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    driver.saveDriverScript(script_create.script.name, script_create.script.content)

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def delete_driver_script(driver, script_delete=None):  # noqa: E501
    """Delete a script

    Delete a script # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param script_delete: The data needed to delete this script
    :type script_delete: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        script_delete = ScriptDelete.from_dict(connexion.request.get_json())  # noqa: E501

    response = errorIfUnauthorized(role='developer')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    driver.deleteDriverScript(script_delete.script.name)

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def get_driver_script(driver, script=None):  # noqa: E501
    """Retrieve the contents of a script

    Retrieve the contents of a script # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param script: The script name.
    :type script: str

    :rtype: Response
    """

    response = errorIfUnauthorized(role='user')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)

    response.body.add({'content': driver.getDriverScript(script)})

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def get_driver_script_catalog(driver):  # noqa: E501
    """Retrieve the script catalog

    Retrieve the script catalog # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """

    response = errorIfUnauthorized(role='user')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)

    response.body.add(driver.getDriverScriptCatalog().get())

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def rename_driver_script(driver, script_rename=None):  # noqa: E501
    """Rename a script

    Rename a script # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param script_rename: The data needed to rename this script
    :type script_rename: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        script_rename = ScriptRename.from_dict(connexion.request.get_json())  # noqa: E501

    response = errorIfUnauthorized(role='developer')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    if not driver.renameDriverScript(script_rename.original.name, script_rename.new.name):
        return ErrorResponse(status=500, message='Cannot rename to an existing file.')

    return Response(status=200, body=response.getResponseBody())

@jwt_required
def save_driver_script(driver, script_save=None):  # noqa: E501
    """Save a script

    Save a script # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param script_save: The data needed to save this script
    :type script_save: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        script_save = ScriptSave.from_dict(connexion.request.get_json())  # noqa: E501

    response = errorIfUnauthorized(role='developer')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    driver.saveDriverScript(script_save.script.name, script_save.script.content)

    return Response(status=200, body=response.getResponseBody())



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from apitax.api.models.base_model_ import Model
from apitax.api import util


class Command(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, parameters=None, options=None, command=None):  # noqa: E501
        """Command - a model defined in Swagger

        :param parameters: The parameters of this Command.  # noqa: E501
        :type parameters: object
        :param options: The options of this Command.  # noqa: E501
        :type options: object
        :param command: The command of this Command.  # noqa: E501
        :type command: str
        """
        self.swagger_types = {
            'parameters': object,
            'options': object,
            'command': str
        }

        self.attribute_map = {
            'parameters': 'parameters',
            'options': 'options',
            'command': 'command'
        }

        self._parameters = parameters
        self._options = options
        self._command = command

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Command of this Command.  # noqa: E501
        :rtype: Command
        """
        return util.deserialize_model(dikt, cls)

    @property
    def parameters(self):
        """Gets the parameters of this Command.


        :return: The parameters of this Command.
        :rtype: object
        """
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        """Sets the parameters of this Command.


        :param parameters: The parameters of this Command.
        :type parameters: object
        """

        self._parameters = parameters

    @property
    def options(self):
        """Gets the options of this Command.


        :return: The options of this Command.
        :rtype: object
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this Command.


        :param options: The options of this Command.
        :type options: object
        """
        if options is None:
            raise ValueError("Invalid value for `options`, must not be `None`")  # noqa: E501

        self._options = options

    @property
    def command(self):
        """Gets the command of this Command.


        :return: The command of this Command.
        :rtype: str
        """
        return self._command

    @command.setter
    def command(self, command):
        """Sets the command of this Command.


        :param command: The command of this Command.
        :type command: str
        """
        if command is None:
            raise ValueError("Invalid value for `command`, must not be `None`")  # noqa: E501

        self._command = command



class Project:
    def isDev(self):
        return True

    def isDebug(self):
        return True

    def setup(self):
        from apitaxcore.models.State import State
        #State.paths['root'] = 'D:/Programming/Projects/Apitax/Apitax'  # just /app for docker
        #State.paths['config'] = State.paths['root'] + '/config.txt'
        #State.paths['log'] = None

    def loadDrivers(self):
        from apitaxcore.drivers.Drivers import Drivers
        #from apitaxdrivers.Github import GithubDriver
        #Drivers.add("github", GithubDriver())



import connexion
import six

from apitax.api.models.auth_response import AuthResponse  # noqa: E501
from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api.models.user_auth import UserAuth  # noqa: E501
from apitax.api.models.user_create import UserCreate  # noqa: E501
from apitax.api.models.user_delete import UserDelete  # noqa: E501
from apitax.api.models.user_save import UserSave  # noqa: E501
from apitax.api import util

from apitax.authentication.ApitaxAuthentication import ApitaxAuthentication
from apitax.api.utilities.Mappers import mapUserAuthToCredentials, mapUserToUser
from apitaxcore.models.User import User

from apitaxcore.flow.LoadedDrivers import LoadedDrivers
from apitaxcore.drivers.Driver import Driver
from apitaxcore.flow.responses.ApitaxResponse import ApitaxResponse
from apitax.api.utilities.Lifecycle import redirectIfUnauthorized, errorIfUnauthorized
from flask_jwt_extended import (create_access_token, create_refresh_token, jwt_required, jwt_refresh_token_required,
                                get_jwt_identity, get_raw_jwt, get_jwt_claims)


def authenticate(user=None):  # noqa: E501
    """Authenticate

    Authenticate with the API # noqa: E501

    :param user: The user authentication object.
    :type user: dict | bytes

    :rtype: AuthResponse
    """
    if connexion.request.is_json:
        user = UserAuth.from_dict(connexion.request.get_json())  # noqa: E501

    credentials = mapUserAuthToCredentials(user)
    auth = ApitaxAuthentication.login(credentials)

    if not auth:
        return ErrorResponse(status=401, message="Invalid credentials")

    access_token = create_access_token(identity={'username': user.username, 'role': auth['role']})
    refresh_token = create_refresh_token(identity={'username': user.username, 'role': auth['role']})

    return AuthResponse(status=201, message='User ' + user.username + ' was authenticated as ' + auth['role'],
                        access_token=access_token, refresh_token=refresh_token,
                        auth=UserAuth(username=auth['credentials'].username, api_token=auth['credentials'].token))


@jwt_required
def create_user(user, driver, user_create=None):  # noqa: E501
    """Create a new user

    Create a new user # noqa: E501

    :param user: Create user with this name
    :type user: str
    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param user_create: The data needed to create this user
    :type user_create: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        user_create = UserCreate.from_dict(connexion.request.get_json())  # noqa: E501

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    user: User = mapUserToUser(user_create.script)

    if driver.createApitaxUser(user):
        return Response(status=200, body=response.getResponseBody())

    return ErrorResponse(status=500, message='Failed to create user')


@jwt_required
def delete_user(user, driver, user_delete=None):  # noqa: E501
    """Delete a user

    Delete a user # noqa: E501

    :param user: Delete user with this name
    :type user: str
    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param user_delete: The data needed to delete this user
    :type user_delete: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        user_delete = UserDelete.from_dict(connexion.request.get_json())  # noqa: E501

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)

    if driver.deleteApitaxUser(User(username=user)):
        return Response(status=200, body=response.getResponseBody())

    return ErrorResponse(status=500, message='Failed to create user')


@jwt_required
def get_config():  # noqa: E501
    """Retrieve the system config

    Retrieve the system config # noqa: E501


    :rtype: Response
    """
    # TODO: Implement
    return 'do some magic!'


@jwt_required
def get_log(log):  # noqa: E501
    """Retrieve the logs

    Retrieve the logs # noqa: E501

    :param log: Get this log
    :type log: str

    :rtype: Response
    """
    # TODO: Implement
    return 'do some magic!'


@jwt_required
def get_user(user, driver):  # noqa: E501
    """Retrieve a user

    Retrieve a user # noqa: E501

    :param user: Get user with this name
    :type user: str
    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """

    response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    user: User = driver.getApitaxUser(User(username=user))
    response.body.add({'user': {'username': user.username, 'role': user.role}})

    return Response(status=200, body=response.getResponseBody())


@jwt_required
def get_user_list(driver):  # noqa: E501
    """Retrieve a list of users

    Retrieve a list of users # noqa: E501

    :param driver: The driver to use for the request. ie. github
    :type driver: str

    :rtype: Response
    """
    # TODO: This requires an additional driver function
    return 'do some magic!'


@jwt_refresh_token_required
def refresh_token():  # noqa: E501
    """Refreshes login token using refresh token

    Refreshes login token using refresh token # noqa: E501


    :rtype: UserAuth
    """
    current_user = get_jwt_identity()
    if not current_user:
        return ErrorResponse(status=401, message="Not logged in")
    access_token = create_access_token(identity=current_user)
    return AuthResponse(status=201, message='Refreshed Access Token', access_token=access_token, auth=UserAuth())


@jwt_required
def save_user(user, driver, user_save=None):  # noqa: E501
    """Save a user

    Save a user # noqa: E501

    :param user: Save user with this name
    :type user: str
    :param driver: The driver to use for the request. ie. github
    :type driver: str
    :param user_save: The data needed to save this user
    :type user_save: dict | bytes

    :rtype: Response
    """
    if connexion.request.is_json:
        user_save = UserSave.from_dict(connexion.request.get_json())  # noqa: E501

    response = errorIfUnauthorized(role='admin')
    if response:
        return response
    else:
        response = ApitaxResponse()

    driver: Driver = LoadedDrivers.getDriver(driver)
    user: User = mapUserToUser(user_save.script)

    if driver.saveApitaxUser(user):
        return Response(status=200, body=response.getResponseBody())

    return ErrorResponse(status=500, message='Failed to create user')


@jwt_required
def system_status():  # noqa: E501
    """Retrieve the system status

    Retrieve the system status # noqa: E501


    :rtype: Response
    """
    # TODO: Implement
    # Return log config, security config, apitax config, IP, port, loaded drivers and used drivers
    # Return primary driver, auth driver etc.
    # Return statistics
    return 'do some magic!'



# Application import
from scriptax.parser.utils.BoilerPlate import customizableParser
from scriptax.drivers.builtin.Scriptax import Scriptax
from commandtax.drivers.builtin.Commandtax import Commandtax
from scriptaxstd.drivers.builtin.StandardLibrary import StandardLibrary
from scriptaxstd.drivers.builtin.Api import Api
from scriptaxstd.drivers.builtin.ApiXml import ApiXml

from apitaxcore.logs.Log import Log
from apitaxcore.logs.StandardLog import StandardLog
from apitaxcore.models.State import State
from apitaxcore.flow.LoadedDrivers import LoadedDrivers
from apitaxcore.drivers.Drivers import Drivers

State.log = Log(StandardLog(), logColorize=False)

#State.log.log("> test")

Drivers.add("commandtax", Commandtax())
Drivers.add("scriptax", Scriptax())
Drivers.add("api", Api())
Drivers.add("apixml", ApiXml())
Drivers.add("std", StandardLibrary())
LoadedDrivers.load("commandtax")
LoadedDrivers.load("scriptax")
LoadedDrivers.load("api")
LoadedDrivers.load("apixml")
LoadedDrivers.load("std")

scriptax = "from std import String as String; log(String.substr(text='this_is_some_fantastic_text', start=3, length=10));"

visitor = customizableParser(scriptax, file='inline_program')

print('Return: ' + str(visitor[0][1]))
print()
print("===")



from apitaxcore.models.Credentials import Credentials
from apitaxcore.models.User import User
from apitax.api.models.user_auth import UserAuth
from apitax.api.models.user import User as UserModel


def mapUserAuthToCredentials(userAuth: UserAuth, credentials=None):
    if not credentials:
        credentials = Credentials()

    if userAuth.api_token:
        credentials.token = userAuth.api_token
    else:
        credentials.username = userAuth.username
        credentials.password = userAuth.password

    if userAuth.extra:
        credentials.extra = userAuth.extra

    return credentials


def mapUserToUser(userModel: UserModel, user: User = None):
    if not user:
        user = User()

    if userModel.username:
        user.username = userModel.username

    if userModel.password:
        user.password = userModel.password

    if userModel.role:
        user.role = userModel.role

    return user




from apitaxcore.flow.responses.ApitaxResponse import ApitaxResponse

from scriptaxstd.drivers.builtin.Api import Api

from commandtax.models.Command import Command


class Apitax(Api):
    def getDriverName(self) -> str:
        return "Apitax"

    def getDriverDescription(self) -> str:
        return "Provides out of the box commandtax support for the Apitax API"

    def getDriverHelpEndpoint(self) -> str:
        return "coming soon"

    def getDriverTips(self) -> str:
        return "coming soon"

    def isDriverCommandable(self) -> bool:
        return True

    def handleDriverCommand(self, command: Command) -> ApitaxResponse:
        return super().handleDriverCommand(command) # TODO: Implement a command parser prior to calling super



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from apitax.api.models.base_model_ import Model
from apitax.api import util


class UserAuth(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, username=None, password=None, api_token=None, extra=None):  # noqa: E501
        """UserAuth - a model defined in Swagger

        :param username: The username of this UserAuth.  # noqa: E501
        :type username: str
        :param password: The password of this UserAuth.  # noqa: E501
        :type password: str
        :param api_token: The api_token of this UserAuth.  # noqa: E501
        :type api_token: str
        :param extra: The extra of this UserAuth.  # noqa: E501
        :type extra: object
        """
        self.swagger_types = {
            'username': str,
            'password': str,
            'api_token': str,
            'extra': object
        }

        self.attribute_map = {
            'username': 'username',
            'password': 'password',
            'api_token': 'api_token',
            'extra': 'extra'
        }

        self._username = username
        self._password = password
        self._api_token = api_token
        self._extra = extra

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The UserAuth of this UserAuth.  # noqa: E501
        :rtype: UserAuth
        """
        return util.deserialize_model(dikt, cls)

    @property
    def username(self):
        """Gets the username of this UserAuth.


        :return: The username of this UserAuth.
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this UserAuth.


        :param username: The username of this UserAuth.
        :type username: str
        """

        self._username = username

    @property
    def password(self):
        """Gets the password of this UserAuth.


        :return: The password of this UserAuth.
        :rtype: str
        """
        return self._password

    @password.setter
    def password(self, password):
        """Sets the password of this UserAuth.


        :param password: The password of this UserAuth.
        :type password: str
        """

        self._password = password

    @property
    def api_token(self):
        """Gets the api_token of this UserAuth.


        :return: The api_token of this UserAuth.
        :rtype: str
        """
        return self._api_token

    @api_token.setter
    def api_token(self, api_token):
        """Sets the api_token of this UserAuth.


        :param api_token: The api_token of this UserAuth.
        :type api_token: str
        """

        self._api_token = api_token

    @property
    def extra(self):
        """Gets the extra of this UserAuth.


        :return: The extra of this UserAuth.
        :rtype: object
        """
        return self._extra

    @extra.setter
    def extra(self, extra):
        """Sets the extra of this UserAuth.


        :param extra: The extra of this UserAuth.
        :type extra: object
        """

        self._extra = extra



# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from apitax.api.models.error_response import ErrorResponse  # noqa: E501
from apitax.api.models.response import Response  # noqa: E501
from apitax.api.test import BaseTestCase


class TestDriversController(BaseTestCase):
    """DriversController integration test stubs"""

    def test_get_driver_blacklist(self):
        """Test case for get_driver_blacklist

        Retrieve the blacklist in the driver
        """
        response = self.client.open(
            '/apitax/2/drivers/{driver}/blacklist'.format(driver='driver_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_driver_config(self):
        """Test case for get_driver_config

        Retrieve the config of a loaded driver
        """
        response = self.client.open(
            '/apitax/2/drivers/{driver}/config'.format(driver='driver_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_driver_list(self):
        """Test case for get_driver_list

        Retrieve the catalog of drivers
        """
        response = self.client.open(
            '/apitax/2/drivers',
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_driver_status(self):
        """Test case for get_driver_status

        Retrieve the status of a loaded driver
        """
        response = self.client.open(
            '/apitax/2/drivers/{driver}/status'.format(driver='driver_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_driver_whitelist(self):
        """Test case for get_driver_whitelist

        Retrieve the whitelist in the driver
        """
        response = self.client.open(
            '/apitax/2/drivers/{driver}/whitelist'.format(driver='driver_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()
